#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Client class for connecting to Logitech Harmony devices."""

import json
import time
import sleekxmpp
from sleekxmpp.xmlstream import ET
import logging

logger = logging.getLogger(__name__)

class HarmonyClient(sleekxmpp.ClientXMPP):
    """An XMPP client for connecting to the Logitech Harmony devices."""

    def __init__(self, auth_token):
        user = '%s@connect.logitech.com/gatorade.' % auth_token
        password = auth_token
        plugin_config = {
            # Enables PLAIN authentication which is off by default.
            'feature_mechanisms': {'unencrypted_plain': True},
        }
        super(HarmonyClient, self).__init__(
            user, password, plugin_config=plugin_config)

    def get_config(self):
        """Retrieves the Harmony device configuration.

        Returns:
            A nested dictionary containing activities, devices, etc.
        """
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = (
            'vnd.logitech.harmony/vnd.logitech.harmony.engine?config')
        iq_cmd.set_payload(action_cmd)
        try:
            result = iq_cmd.send(block=True)
        except Exception:
            logger.info('XMPP timeout, reattempting')
            result = iq_cmd.send(block=True)
        payload = result.get_payload()
        assert len(payload) == 1
        action_cmd = payload[0]
        assert action_cmd.attrib['errorcode'] == '200'
        device_list = action_cmd.text
        return json.loads(device_list)

    def get_current_activity(self):
        """Retrieves the current activity ID.

        Returns:
            A int with the current activity ID.
        """
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = (
            'vnd.logitech.harmony/vnd.logitech.harmony.engine?getCurrentActivity')
        iq_cmd.set_payload(action_cmd)
        try:
            result = iq_cmd.send(block=True)
        except Exception:
            logger.info('XMPP timeout, reattempting')
            result = iq_cmd.send(block=True)
        payload = result.get_payload()
        assert len(payload) == 1
        action_cmd = payload[0]
        assert action_cmd.attrib['errorcode'] == '200'
        activity = action_cmd.text.split("=")
        return int(activity[1])

    def start_activity(self, activity_id):
        """Starts an activity.

        Args:
            activity_id: An int or string identifying the activity to start

        Returns:
            True if activity started, otherwise False
        """
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = ('harmony.engine?startactivity')
        cmd = 'activityId=' + str(activity_id) + ':timestamp=0'
        action_cmd.text = cmd
        iq_cmd.set_payload(action_cmd)
        try:
            result = iq_cmd.send(block=True)
        except Exception:
            logger.info('XMPP timeout, reattempting')
            result = iq_cmd.send(block=True)
        payload = result.get_payload()
        assert len(payload) == 1
        action_cmd = payload[0]
        if action_cmd.text == None:
            return True
        else:
            return False

    def sync(self):
        """Syncs the harmony hub with the web service.
        """
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = ('setup.sync')
        iq_cmd.set_payload(action_cmd)
        try:
            result = iq_cmd.send(block=True)
        except Exception:
            logger.info('XMPP timeout, reattempting')
            result = iq_cmd.send(block=True)
        payload = result.get_payload()
        assert len(payload) == 1

    def send_command(self, device, command):
        """Send a simple command to the Harmony Hub.

        Args:
            device_id (str): Device ID from Harmony Hub configuration to control
            command (str): Command from Harmony Hub configuration to control

        Returns:
            None if successful
        """
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        iq_cmd['id'] = '5e518d07-bcc2-4634-ba3d-c20f338d8927-2'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = (
            'vnd.logitech.harmony/vnd.logitech.harmony.engine?holdAction')
        action_cmd.text = 'action={"type"::"IRCommand","deviceId"::"' + device + '","command"::"' + command + '"}:status=press'
        iq_cmd.set_payload(action_cmd)
        result = iq_cmd.send(block=False)

        action_cmd.attrib['mime'] = (
            'vnd.logitech.harmony/vnd.logitech.harmony.engine?holdAction')
        action_cmd.text = 'action={"type"::"IRCommand","deviceId"::"' + device + '","command"::"' + command + '"}:status=release'
        iq_cmd.set_payload(action_cmd)
        result = iq_cmd.send(block=False)
        return result

    def power_off(self):
        """Turns the system off if it's on, otherwise it does nothing.

        Returns:
            True
        """
        activity = self.get_current_activity()
        if activity != -1:
            if self.start_activity(-1):
                return True
            else:
                return False


def create_and_connect_client(ip_address, port, token):
    """Creates a Harmony client and initializes session.

    Args:
        ip_address (str): Harmony device IP address
        port (str): Harmony device port
        token (str): Valid session token

    Returns:
        A connected HarmonyClient instance
    """
    client = HarmonyClient(token)
    client.connect(address=(ip_address, port),
                   use_tls=False, use_ssl=False)
    client.process(block=False)
    while not client.sessionstarted:
        time.sleep(0.1)
    return client



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (c) 2013, Jeff Terrace
# All rights reserved.

"""Authentication routines to conect to Logitech web service and Harmony devices."""

import json
import logging
import re
import requests
import sleekxmpp
from sleekxmpp.xmlstream import ET

logger = logging.getLogger(__name__)

#Logitech authentication service URL.
LOGITECH_AUTH_URL = ('https://svcs.myharmony.com/CompositeSecurityServices/'
                     'Security.svc/json/GetUserAuthToken')


def login(username, password):
    """Logs in to the Logitech Harmony web service.

    Args:
        username (str): The username (email address).
        password (str): The user's password.

    Returns:
        A base64-encoded string containing a 48-byte Login Token.
    """
    headers = {'content-type': 'application/json; charset=utf-8'}
    data = {'email': username, 'password': password}
    data = json.dumps(data)
    resp = requests.post(LOGITECH_AUTH_URL, headers=headers, data=data)
    if resp.status_code != 200:
        logger.error('Received response code %d from Logitech.',
                     resp.status_code)
        logger.error('Data: \n%s\n', resp.text)
        return

    result = resp.json().get('GetUserAuthTokenResult', None)
    if not result:
        logger.error('Malformed JSON (GetUserAuthTokenResult): %s', resp.json())
        return
    token = result.get('UserAuthToken', None)
    if not token:
        logger.error('Malformed JSON (UserAuthToken): %s', resp.json())
        return
    return token


class SwapAuthToken(sleekxmpp.ClientXMPP):
    """An XMPP client for swapping a Login Token for a Session Token.

    After the client finishes processing, the uuid attribute of the class will
    contain the session token.
    """

    def __init__(self, token):
        """Initializes the client.

        Args:
          token: The base64 string containing the 48-byte Login Token.

        """
        plugin_config = {
            # Enables PLAIN authentication which is off by default.
            'feature_mechanisms': {'unencrypted_plain': True},
        }
        super(SwapAuthToken, self).__init__(
            'guest@connect.logitech.com/gatorade.', 'gatorade.', plugin_config=plugin_config)

        self.token = token
        self.uuid = None
        self.add_event_handler('session_start', self.session_start)

    def session_start(self, _):
        """Called when the XMPP session has been initialized."""
        iq_cmd = self.Iq()
        iq_cmd['type'] = 'get'
        action_cmd = ET.Element('oa')
        action_cmd.attrib['xmlns'] = 'connect.logitech.com'
        action_cmd.attrib['mime'] = 'vnd.logitech.connect/vnd.logitech.pair'
        action_cmd.text = 'token=%s:name=%s' % (self.token,
                                                'foo#iOS6.0.1#iPhone')
        iq_cmd.set_payload(action_cmd)
        result = iq_cmd.send(block=True)
        payload = result.get_payload()
        assert len(payload) == 1
        oa_resp = payload[0]
        assert oa_resp.attrib['errorcode'] == '200'
        match = re.search(r'identity=(?P<uuid>[\w-]+):status', oa_resp.text)
        assert match
        self.uuid = match.group('uuid')
        logger.info('Received UUID from device: %s', self.uuid)
        self.disconnect(send_close=False)


def swap_auth_token(ip_address, port, token):
    """Swaps the Logitech auth token for a session token.

    Args:
        ip_address (str): IP Address of the Harmony device IP address
        port (str): Harmony device port
        token (str): A base64-encoded string containing a 48-byte Login Token.

    Returns:
        A string containing the session token.
    """
    login_client = SwapAuthToken(token)
    login_client.connect(address=(ip_address, port),use_tls=False, use_ssl=False)
    login_client.process(block=True)
    return login_client.uuid



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys

here = lambda *a: os.path.join(os.path.dirname(__file__), *a)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open(here('README.md')).read()
requirements = [x.strip() for x in open(here('requirements.txt')).readlines()]

setup(
    name='pyharmony',
    version='1.0.6',
    description='Python library for programmatically using a Logitech Harmony Link or Ultimate Hub.',
    long_description=readme,
    author='Ian Day',
    author_email='iandday@gmail.com',
    url='https://github.com/iandday/pyharmony',
    download_url = 'https://github.com/iandday/pyharmony/tarball/1.0.6',
    packages=['pyharmony'],
    package_dir={'pyharmony': 'pyharmony'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='pyharmony',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Topic :: Home Automation',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.5'
    ],
    entry_points={
        'console_scripts': [
            'harmony = pyharmony.__main__:main'
        ]
    },
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Module for querying and controlling Logitech Harmony devices."""

import argparse
import json
import logging
from pyharmony import auth as harmony_auth
from pyharmony import client as harmony_client
import sys
import time

logger = logging.getLogger(__name__)
logger.setLevel(logging.ERROR)

# Trim down log file spam
logging.getLogger('sleekxmpp').setLevel(logging.CRITICAL)
logging.getLogger('requests').setLevel(logging.CRITICAL)
logging.getLogger('pyharmony').setLevel(logging.CRITICAL)


def pprint(obj):
    """Pretty JSON dump of an object."""
    print(json.dumps(obj, sort_keys=True, indent=4, separators=(',', ': ')))


def login_to_logitech(email, password, harmony_ip, harmony_port):
    """Logs in to the Logitech service.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        Session token used to log in to the Harmony device.
    """
    token = harmony_auth.login(email, password)
    if not token:
        logger.warning('Could not get token from Logitech server.')

    session_token = harmony_auth.swap_auth_token(harmony_ip, harmony_port, token)
    if not session_token:
        logger.warning('Could not swap login token for session token.')

    return session_token


def get_client(email, password, harmony_ip, harmony_port):
    """Connect to the Harmony and return a Client instance.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        object: Authenticated client instance.
    """
    token = login_to_logitech(email, password, harmony_ip, harmony_port)
    client = harmony_client.create_and_connect_client(harmony_ip, harmony_port, token)
    return client


# Functions for use when module is imported
def ha_get_config(email, password, harmony_ip, harmony_port):
    """Connects to the Harmony and generates a dictionary containing all activites and commands programmed to hub.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        Dictionary containing Harmony device configuration
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    config = client.get_config()
    client.disconnect(send_close=True)
    return config


def ha_get_config_file(email, password, harmony_ip, harmony_port, path):
    """Connects to the Harmony and generates a text file containing all activites and commands programmed to hub.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port
        path (str): Full path to file that function will write activity and command information

    Returns:
        True
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    config = client.get_config()
    client.disconnect(send_close=True)

    with open(path, 'w+') as file_out:
        file_out.write('Activities\n')
        for activity in config['activity']:
            file_out.write('  ' + activity['label'] + '\n')

        file_out.write('\nDevice Commands\n')
        for device in config['device']:
            file_out.write('  ' + device['id'] + ' - ' + device['label'] + '\n')
            for controlGroup in device['controlGroup']:
                for function in controlGroup['function']:
                    file_out.write('    ' + function['name'] + '\n')
    return True


def ha_get_activities(email, password, harmony_ip, harmony_port):
    """Connects to the Harmony hub and returns configured activities.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        Dictionary containing activity label and ID number.
    """
    config = ha_get_config(email, password, harmony_ip, harmony_port)
    activities = {}
    for activity in config['activity']:
        activities[activity['label']] = activity['id']
    if activities != {}:
        return activities
    else:
        logger.error('Unable to hub\'s activities')
        return activities


def ha_get_current_activity(email, password, harmony_ip, harmony_port):
    """Returns Harmony hub's current activity.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        String containing hub's current activity.
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    config = client.get_config()
    current_activity_id = client.get_current_activity()
    client.disconnect(send_close=True)
    activity = [x for x in config['activity'] if int(x['id']) == current_activity_id][0]
    if type(activity) is dict:
        return activity['label']
    else:
        logger.error('Unable to retrieve current activity')
        return 'Unknown'


def ha_start_activity(email, password, harmony_ip, harmony_port, new_activity):
    """Connects to Harmony Hub and starts an activity

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port
        new_actvivity (str): Activity ID or label to start

    Returns:
        True if activity started, otherwise False
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    status = False

    if (new_activity.isdigit()) or (new_activity == '-1'):
        status = client.start_activity(new_activity)
        client.disconnect(send_close=True)
        if status:
            return True
        else:
            logger.info('Activity start failed')
            return False

    else:
        config = client.get_config()
        activities = config['activity']
        labels_and_ids = dict([(a['label'], a['id']) for a in activities])
        matching = [label for label in list(labels_and_ids.keys())
                    if new_activity.lower() in label.lower()]
        if len(matching) == 1:
            activity = matching[0]
            logger.info('Found activity named %s (id %s)' % (activity, labels_and_ids[activity]))
            status = client.start_activity(labels_and_ids[activity])
        client.disconnect(send_close=True)
        if status:
            return True
        else:
            logger.error('Unable to find matching activity, start failed %s' % (' '.join(matching)))
            return False


def ha_power_off(email, password, harmony_ip, harmony_port):
    """Power off Harmony Hub.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        True if PowerOff activity started, otherwise False

    """
    client = get_client(email, password, harmony_ip, harmony_port)
    status = client.power_off()
    client.disconnect(send_close=True)
    if status:
        return True
    else:
        logger.error('Power Off failed')
        return False


def ha_send_command(email, password, harmony_ip, harmony_port, device, command):
    """Connects to the Harmony and send a simple command.

    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port
        device_id (str): Device ID from Harmony Hub configuration to control
        new_command (str): Command from Harmony Hub configuration to control

    Returns:
        Completion status
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    client.send_command(device, command)
    time.sleep(1)
    client.disconnect(send_close=True)
    return 0


def ha_sync(email, password, harmony_ip, harmony_port):
    """Syncs Harmony hub to web service.
    Args:
        email (str):  Email address used to login to Logitech service
        password (str): Password used to login to Logitech service
        harmony_ip (str): Harmony hub IP address
        harmony_port (str): Harmony hub port

    Returns:
        Completion status
    """
    client = get_client(email, password, harmony_ip, harmony_port)
    client.sync()
    client.disconnect(send_close=True)
    return 0


# Functions for use on command line
def show_config(args):
    """Connects to the Harmony and return current configuration.

    Args:
        args (argparse): Argparse object containing required variables from command line

    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    config = client.get_config()
    client.disconnect(send_close=True)
    pprint(config)


def show_current_activity(args):
    """Returns Harmony hub's current activity.

    Args:
        args (argparse): Argparse object containing required variables from command line

    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    config = client.get_config()
    current_activity_id = client.get_current_activity()
    client.disconnect(send_close=True)
    activity = [x for x in config['activity'] if int(x['id']) == current_activity_id][0]
    if type(activity) is dict:
        print(activity['label'])
    else:
        logger.error('Unable to retrieve current activity')
        print('Unknown')


def start_activity(args):
    """Connects to Harmony Hub and starts an activity

    Args:
        args (argparse): Argparse object containing required variables from command line

    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    status = False

    if (args.activity.isdigit()) or (args.activity == '-1'):
        status = client.start_activity(args.activity)
        client.disconnect(send_close=True)
        if status:
            print('Started Actvivity')
        else:
            logger.info('Activity start failed')
    else:
        config = client.get_config()
        activities = config['activity']
        labels_and_ids = dict([(a['label'], a['id']) for a in activities])
        matching = [label for label in list(labels_and_ids.keys())
                    if args.activity.lower() in label.lower()]
        if len(matching) == 1:
            activity = matching[0]
            logger.info('Found activity named %s (id %s)' % (activity, labels_and_ids[activity]))
            status = client.start_activity(labels_and_ids[activity])
        client.disconnect(send_close=True)
        if status:
            print('Started:', args.activity)
        else:
            logger.error('found too many activities! %s' % (' '.join(matching)))


def power_off(args):
    """Power off Harmony Hub.

    Args:
        args (argparse): Argparse object containing required variables from command line

    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    status = client.power_off()
    client.disconnect(send_close=True)
    if status:
        print('Powered Off')
    else:
        logger.error('Power off failed')


def send_command(args):
    """Connects to the Harmony and send a simple command.

    Args:
        args (argparse): Argparse object containing required variables from command line

    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    client.send_command(args.device_id, args.command)
    client.disconnect(send_close=True)
    print('Command Sent')


def sync(args):
    """Syncs Harmony hub to web service.
    Args:
        args (argparse): Argparse object containing required variables from command line

    Returns:
        Completion status
    """
    client = get_client(args.email, args.password, args.harmony_ip, args.harmony_port)
    client.sync()
    client.disconnect(send_close=True)
    print('Sync complete')


def main():
    """Main method for the script."""
    parser = argparse.ArgumentParser(description='Pyharmony - Harmony device control',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    required_flags = parser.add_argument_group('required arguments')

    # Required flags go here.
    required_flags.add_argument('--email',
                                required=True,
                                help=('Logitech username in the form of an email address.'))
    required_flags.add_argument('--password',
                                required=True,
                                help='Logitech password.')
    required_flags.add_argument('--harmony_ip',
                                required=True,
                                help='IP Address of the Harmony device.')

    # Flags with default values go here.
    loglevels = dict((logging.getLevelName(level), level) for level in [10, 20, 30, 40, 50])
    parser.add_argument('--harmony_port',
                        required=True,
                        default=5222,
                        type=int,
                        help=('Network port that the Harmony is listening on.'))
    parser.add_argument('--loglevel',
                        default='INFO',
                        choices=list(loglevels.keys()),
                        help='Logging level to print to the console.')

    subparsers = parser.add_subparsers()

    show_config_parser = subparsers.add_parser('show_config', help='Print the Harmony device configuration.')
    show_config_parser.set_defaults(func=show_config)

    show_activity_parser = subparsers.add_parser('show_current_activity', help='Print the current activity config.')
    show_activity_parser.set_defaults(func=show_current_activity)

    start_activity_parser = subparsers.add_parser('start_activity', help='Switch to a different activity.')
    start_activity_parser.add_argument('--activity', help='Activity to switch to, id or label.')
    start_activity_parser.set_defaults(func=start_activity)

    power_off_parser = subparsers.add_parser('power_off', help='Stop the activity.')
    power_off_parser.set_defaults(func=power_off)

    sync_parser = subparsers.add_parser('sync', help='Sync the harmony.')
    sync_parser.set_defaults(func=sync)

    command_parser = subparsers.add_parser('send_command', help='Send a simple command.')
    command_parser.add_argument('--device_id', help='Specify the device id to which we will send the command.')
    command_parser.add_argument('--command', help='IR Command to send to the device.')
    command_parser.set_defaults(func=send_command)

    args = parser.parse_args()

    logging.basicConfig(
        level=loglevels[args.loglevel],
        format='%(levelname)s:\t%(name)s\t%(message)s')

    sys.exit(args.func(args))


if __name__ == '__main__':
    main()
